/*
 *
 *  Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved. 
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  Further, this software is distributed without any warranty that it is
 *  free of the rightful claim of any third person regarding infringement
 *  or the like.  Any license provided herein, whether implied or
 *  otherwise, applies only to this software file.  Patent licenses, if
 *  any, provided herein do not apply to combinations of this program with
 *  other software, or any other product whatsoever.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
 *  Mountain View, CA  94043, or:
 * 
 *  http://www.sgi.com 
 * 
 *  For further information regarding this notice, see: 
 * 
 *  http://oss.sgi.com/projects/GenInfo/NoticeExplan/
 *
 */

/*
 * Copyright (C) 1989,1990,1991   Silicon Graphics, Inc.
 *
 ***********************************************************************
 **************  S I L I C O N   G R A P H I C S   I N C .  ************
 *
 *   File Name	: MMwrite.c
 *
 *   Description:
 *	This writes the basic Makefile templates.
 *
 *   Revision	: $Revision: 1.1.1.1 $
 *   Author	: David Mott
 *
 **************  S I L I C O N   G R A P H I C S   I N C .  ************
 ***********************************************************************
 */

#include <stdio.h>
#include <string.h>

#define OUT(STRING) fprintf(makefile,"%s\n",STRING)

static void
writeHeader(FILE *makefile, char *filename)
{
    OUT("#!smake");
    OUT("# Copyright (C) 1990,1991  Silicon Graphics, Inc.");
    OUT("#");
    OUT("#_______________________________________________________________________");
    OUT("#______________  S I L I C O N   G R A P H I C S   I N C .  ____________");
    OUT("#");
    OUT(filename);
    OUT("#");
    OUT("#                         ******* NOTE *******");
    OUT("#");
    OUT("#   	This file was generated by makemake - do not edit by hand.");
    OUT("#");
    OUT("#                         ******* NOTE *******");
    OUT("#");
    OUT("#______________  S I L I C O N   G R A P H I C S   I N C .  ____________");
    OUT("#_______________________________________________________________________");
    OUT("#");
}

void
MMwrite_libMakefile(FILE *makefile, char *name, char *srcdirs, char *incldirs)
{
    char filename[256];
    sprintf(filename, "#   File Name: Inventor/lib/%s/Makefile", name);

    writeHeader(makefile, filename);
    OUT(" ");
    OUT("DEPTH = ../");
    OUT("include $(DEPTH)../make/so_commondefs");
    OUT(" ");
    OUT("# specific to this library");
    fprintf(makefile, "LIBNAME    = lib%s.a\n", name);
    fprintf(makefile, "COMMONPREF = lib%s\n", name);
    fprintf(makefile, "NAME       = %s\n", name);
    fprintf(makefile, "SRCDIRS    = %s\n", srcdirs);
    fprintf(makefile, "INCLDIRS	  = %s\n", incldirs);
    OUT(" ");
    OUT("include $(SO_LIBCOMMONRULES)");
}

void
MMwrite_libSrcMakefile(FILE *makefile, char *name, char *dir, char *depth)
{
    char filename[256];
    sprintf(filename, "#   File Name: Inventor/lib/%s/%s/Makefile", name, dir);

    writeHeader(makefile, filename);
    OUT(" ");
    fprintf(makefile, "DEPTH = %s\n", depth);
    fprintf(makefile, "SOLIB = $(DEPTH)$(SHLIB_PREFIX)%s/lib%s.a\n", name,name);
    OUT("include $(DEPTH)../make/so_commondefs");
}

/* This one called before HFILES is defined */
void
MMwrite_includeMakefile(FILE *makefile, char *name, char *dir, char *depth)
{
    char filename[256];
    char cppdir[256];
    char cdir[256];
    char *s, *t;
    char oldch;

    sprintf(filename, "#   File Name: Inventor/lib/%s/%s/Makefile", name, dir);

    writeHeader(makefile, filename);
    OUT("");
    fprintf(makefile, "DEPTH = %s\n", depth);
    fprintf(makefile, "include $(DEPTH)../make/so_commondefs\n");
    fprintf(makefile, "\nHINST_DIR = %s\n", dir);
    fprintf(makefile, "ORIG_DIR = %s%s\n", depth, dir);

    s=strstr(dir, "Inventor");
    if (s==NULL) {
	fprintf(stderr, "MMwrite_includeMakefile(): Bogus!\n");
	exit(1);
    }
    s+=8;
    oldch= *s;
    *s=0;
    strcpy(cdir, dir);
    strcpy(cppdir, dir);
    strcat(cdir, "_c");
    strcat(cppdir, "_c++");
    *s=oldch;
    strcat(cdir, s);
    strcat(cppdir, s);

    fprintf(makefile, "CAPI_HINST_DIR = %s\n", cdir);
    fprintf(makefile, "CPP_DIR = %s%s\n", depth, cppdir);
    fprintf(makefile, "CAPI_DIR = %s%s\n", depth, cdir);
    fprintf(makefile, "CAPI_SOLIB = %slib%s_c.a\n\n", depth, name);
}

void
MMwrite_appsMakefile(FILE *makefile, char *name, char *srcdirs, char *loader)
{
    char filename[256];
    sprintf(filename, "#   File Name: Inventor/apps/%s/Makefile", name);

    writeHeader(makefile, filename);
    OUT(" ");
    OUT("DEPTH = ./");
    OUT("include $(DEPTH)../../make/so_commondefs");
    OUT(" ");
    OUT("# specific to this app");
    fprintf(makefile, "PROGNAME   = %s\n", name);
    fprintf(makefile, "COMMONPREF = %s\n", name);
    fprintf(makefile, "NAME       = %s\n", name);
    fprintf(makefile, "SRCDIRS    = %s\n", srcdirs);
    fprintf(makefile, "LOADER     = %s\n", loader);
    OUT(" ");
    OUT("include $(SO_LIBCOMMONRULES)");
}

void
MMwrite_appsSrcMakefile(FILE *makefile, char *name, char *dir, char *depth)
{
    char filename[256];
    sprintf(filename, "#   File Name: Inventor/apps/%s/%s/Makefile", name, dir);

    writeHeader(makefile, filename);
    OUT(" ");
    fprintf(makefile, "DEPTH = %s\n", depth);
    OUT("include $(DEPTH)../../make/so_commondefs");
    OUT("include $(SO_COMMONRULES)");
}

