/**************************************************************************
 *
 *  This file is part of the Coin GUI binding libraries.
 *  Copyright (C) 2000 by Systems in Motion.  All rights reserved.
 *
 *  The libraries this file is part of is free software; you can
 *  redistribute them and/or modify them under the terms of the GNU
 *  Lesser General Public License version 2.1 as published by the
 *  Free Software Foundation.  See the file LICENSE.LGPL at the root
 *  directory of the distribution for all the details.
 *
 *  If you want to use the Coin GUI binding libraries for applications
 *  not compatible with the LGPL, contact SIM about acquiring a
 *  Professional Edition License.
 *
 *  Systems in Motion, Prof Brochs gate 6, N-7030 Trondheim, NORWAY
 *  http://www.sim.no/ support@sim.no Voice: +47 22114160 Fax: +47 22207097
 *
 **************************************************************************/

// Generated automatically from SoGuiViewer.h.in by configure.
// $Id: SoGuiViewer.h.in,v 1.14 2000/09/11 14:58:47 larsa Exp $

#ifndef SOQT_VIEWER_H
#define SOQT_VIEWER_H

/**************************************************************************\
 * 
 *  A WORD OF ADVICE
 *
 *  It is fruitless to modify the contents of the SoQtViewer.h file
 *  because it is autogenerated by configure from the SoAnyViewer.h.in 
 *  file which you will find in the src/Inventor/Qt/common/ directory.
 *  Do your modifications to that file instead.
 *  
\**************************************************************************/

// Workaround for OIV problem on Linux in SbBasic.h.
#include <math.h>

#include <Inventor/SbTime.h>
#include <Inventor/SoType.h>
#include <Inventor/SbLinear.h>

#include <Inventor/Qt/SoQtRenderArea.h>

class SoCallbackList;
class SoBaseColor;
class SoCamera;
class SoComplexity;
class SoDirectionalLight;
class SoDrawStyle;
class SoGetBoundingBoxAction;
class SoGroup;
class SoLightModel;
class SoMaterialBinding;
class SoNode;
class SoPolygonOffset;
class SoSensor;
class SoSeparator;
class SoSwitch;
class SoTimerSensor;

class SoQtViewer;

typedef void SoQtViewerCB(
                 void * data, SoQtViewer * viewer );
typedef void SoQtViewerFPSCB(
                 float fps, void * data, SoQtViewer * viewer );
typedef void SoQtViewerDecimationPercentageCB(
                 float percentage, void * data, SoQtViewer * viewer );

// *************************************************************************

class SOQT_DLL_EXPORT SoQtViewer : public SoQtRenderArea {
  typedef SoQtRenderArea inherited;
  Q_OBJECT

public:
  enum Type {
    BROWSER,
    EDITOR
  };

  enum DecimationStrategy {
    NORMAL,
    FIXED_NUM_TRIANGLES,
    FRAMES_PER_SECOND,
    FIXED_PERCENTAGE
  };

  enum DrawStyle {
    VIEW_AS_IS,
    VIEW_HIDDEN_LINE,
    VIEW_NO_TEXTURE,
    VIEW_LOW_COMPLEXITY,
    VIEW_LINE,
    VIEW_POINT,
    VIEW_BBOX,
    VIEW_LOW_RES_LINE,
    VIEW_LOW_RES_POINT,
    VIEW_SAME_AS_STILL
  };

  enum DrawType {
    STILL = 0,
    INTERACTIVE
  };

  enum BufferType {
    BUFFER_SINGLE,
    BUFFER_DOUBLE,
    BUFFER_INTERACTIVE
  };

  virtual void setCamera( SoCamera * camera );
  SoCamera * getCamera(void) const;

  virtual void setCameraType( SoType type );
  SoType getCameraType(void) const;

  virtual void viewAll(void);

  virtual void saveHomePosition(void);
  virtual void resetToHomePosition(void);

  virtual void setHeadlight( SbBool enable );
  SbBool isHeadlight(void) const;
  SoDirectionalLight * getHeadlight(void) const;

  virtual void setDrawStyle( SoQtViewer::DrawType type, SoQtViewer::DrawStyle style);
  SoQtViewer::DrawStyle getDrawStyle( const SoQtViewer::DrawType type ) const;

  virtual void setBufferingType( SoQtViewer::BufferType type );
  SoQtViewer::BufferType getBufferingType(void) const;

  virtual void setViewing( SbBool enable );
  SbBool isViewing(void) const;

  virtual void setCursorEnabled( SbBool enable );
  SbBool isCursorEnabled(void) const;

  void setAutoClipping( SbBool enable );
  SbBool isAutoClipping(void) const;

  virtual void setStereoViewing( SbBool enable );
  SbBool isStereoViewing(void) const;

  virtual void setStereoOffset( const float dist );
  float getStereoOffset(void) const;

  void setDetailSeek( const SbBool enable );
  SbBool isDetailSeek(void) const;

  void setSeekTime( const float seconds );
  float getSeekTime(void) const;

  void addStartCallback( SoQtViewerCB * func, void * data = NULL );
  void addFinishCallback( SoQtViewerCB * func, void * data = NULL );
  void removeStartCallback( SoQtViewerCB * func, void * data = NULL );
  void removeFinishCallback( SoQtViewerCB * func, void * data = NULL );

  void copyView( const SbTime eventTime );
  void pasteView( const SbTime eventTime );

  virtual void recomputeSceneSize(void);

  void setDecimationStrategy( const DecimationStrategy strategy );
  DecimationStrategy getDecimationStrategy(void) const;

  void setGoalNumberOfTriangles( const int32_t goal );
  int32_t getGoalNumberOfTriangles(void) const;

  void setGoalFramesPerSecond(const float goal);
  float getGoalFramesPerSecond(void) const;

  void setFixedPercentage( const float percent );
  float getFixedPercentage(void) const;

  void enableFullRenderingWhenStill( const SbBool enable );
  SbBool isFullRenderingWhenStill(void) const;
  SbBool isStillNow(void) const;

  void setFramesPerSecondCallback( SoQtViewerFPSCB * callback, void * data );

  void setNumSamples( const int numFrames );
  int getNumSamples(void) const;

  void setDecimationPercentageCallback(
      SoQtViewerDecimationPercentageCB * cb, void * data = NULL );
  float getCurrentDecimationPercentage(void) const;

  // Overloaded from parent class.
  virtual void setDoubleBuffer( SbBool enable );

  virtual void setSceneGraph( SoNode * root );
  virtual SoNode * getSceneGraph(void);

protected:
  SoQtViewer(
    QWidget * parent,
    const char * name,
    SbBool embed,
    Type type,
    SbBool buildNow );
#if SOQT_HAVE_WINWIDGET
  SoQtViewer(
    @WINWIDGET@ parent,
    const char * name,
    SbBool embed,
    Type type,
    SbBool buildNow );
#endif // SOQT_HAVE_WINWIDGET
  ~SoQtViewer( void );

  virtual void sizeChanged( const SbVec2s size );
  virtual void setSeekMode( SbBool enable );
  SbBool isSeekMode(void) const;
  SbBool seekToPoint( const SbVec2s screenpos );

  virtual void actualRedraw(void);
  virtual SbBool processSoEvent( const SoEvent * const event );
  SbBool processCommonEvents( QEvent * event );

  void interactiveCountInc(void);
  void interactiveCountDec(void);
  int getInteractiveCount(void) const;

  void setSeekDistance( const float distance );
  float getSeekDistance(void) const;
  void setSeekValueAsPercentage( const SbBool on );
  SbBool isSeekValuePercentage(void) const;

  virtual void toggleCameraType(void);

protected:
  // FIXME: even more data than this should be protected
  SoCamera * camera;
  SoQtViewer::Type type;
  SbBool viewingFlag;
  SbBool altSwitchBack;

  SoGetBoundingBoxAction * autoClipBboxAction;
  float minimumNearPlane; // % of far plane

  SoNode * sceneGraph; // userroot
  SoSeparator * sceneRoot; // viewerroot

private:
  // Seek functionality
  SoTimerSensor * seeksensor;
  static void seeksensorCB( void * data, SoSensor * );
  float seekperiod;
  SbBool inseekmode;
  SbBool seektopoint;
  SbVec3f camerastartposition, cameraendposition;
  float seekdistance;
  SbBool seekdistanceabs;

  // Camera handling
  SbBool deletecamera;
  SbVec3f storedposition;
  SbRotation storedorientation;
  float storedheightval;

  // Headlight handling
  SbBool lighton;
  SoGroup * lightroot;

  // Drawstyles
  DrawStyle drawstyles[2];
  SbBool drawInteractiveAsStill(void) const;
  SbBool drawAsHiddenLine(void) const;
  SoQtViewer::DrawStyle currentDrawStyle(void) const;
  void changeDrawStyle( SoQtViewer::DrawStyle style );
  SoSwitch * drawstyleroot, * hiddenlineroot, * polygonoffsetparent;
  SoBaseColor * sobasecolor;
  SoComplexity * socomplexity;
  SoDrawStyle * sodrawstyle;
  SoLightModel * solightmodel;
  SoMaterialBinding * somaterialbinding;
  SoPolygonOffset * sopolygonoffset;

  // Automatic setting of clipping planes
  SbBool adjustclipplanes;
  void setClippingPlanes(void);

  // Misc
  SoType cameratype;
  SbBool cursoron;

  SoCallbackList * interactionstartCallbacks, * interactionendCallbacks;

  static void interactivestartCB( void *, SoQtViewer * thisp );
  static void interactiveendCB( void *, SoQtViewer * thisp );
  int interactionnesting;

  BufferType buffertype;

  void moveCameraScreen( const SbVec2f & screenpos );

  SbBool stereoviewing;
  float stereooffset;
}; // class SoQtViewer

// *************************************************************************

#endif // ! SOQT_VIEWER_H
